//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: AntProgramTab.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace AntGP
{
	/// <summary>
    /// This control allows the user to view and create the Programs that are used to control the ants.
    /// The user can view the best program evolved by the Genetic Program, view some canned programs,
    /// or write their own program from scratch.
	/// </summary>
	public class AntProgramTab : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.Label InstructionSetLabel;
		private System.Windows.Forms.ListBox InstructionSetList;
		private System.Windows.Forms.Label GeneticProgramLabel;
		private System.Windows.Forms.TreeView GeneticProgramTreeView;
		private System.Windows.Forms.Label LoadSolutionLabel;
		private System.Windows.Forms.ComboBox LoadSolutionComboBox;
        private System.Windows.Forms.Button AddNewInstructionBeforeButton;
        private System.Windows.Forms.Button RemoveInstructionButton;
        private System.Windows.Forms.Button ChangeInstructionButton;
        private System.Windows.Forms.Button AddNewInstructionAfterButton;
        private System.Windows.Forms.Button ClearProgramButton;
        private System.Windows.Forms.Label WarningMessage;

		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

        /// <summary>Text that is displayed as the root node in the program tree.</summary>
        private static string rootNodeText = "Program Root";

        /// <summary>
        /// Selected node in the program tree. Must be saved outside the tree control, because
        /// the tree loses knowledge of its selection when the user clicks on another control.
        /// </summary>
        TreeNode selectedTreeNode = null;
        
        /// <summary>
        /// The color used by the Program tree to indicate selection. This color is used to change the backgrond color
        /// of the program node because the node is not really selected once another control gets focus.
        /// </summary>
        Color selectColor = Color.FromArgb(49, 106, 197);

        /// <summary>
        /// Default Constructor
        /// </summary>
		public AntProgramTab()
		{
			InitializeComponent();

			PopulateInstructionSetList();

            ResetProgramTree(true);
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.InstructionSetList = new System.Windows.Forms.ListBox();
            this.InstructionSetLabel = new System.Windows.Forms.Label();
            this.GeneticProgramTreeView = new System.Windows.Forms.TreeView();
            this.GeneticProgramLabel = new System.Windows.Forms.Label();
            this.LoadSolutionLabel = new System.Windows.Forms.Label();
            this.LoadSolutionComboBox = new System.Windows.Forms.ComboBox();
            this.AddNewInstructionBeforeButton = new System.Windows.Forms.Button();
            this.RemoveInstructionButton = new System.Windows.Forms.Button();
            this.ChangeInstructionButton = new System.Windows.Forms.Button();
            this.AddNewInstructionAfterButton = new System.Windows.Forms.Button();
            this.ClearProgramButton = new System.Windows.Forms.Button();
            this.WarningMessage = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // InstructionSetList
            // 
            this.InstructionSetList.ItemHeight = 14;
            this.InstructionSetList.Location = new System.Drawing.Point(8, 24);
            this.InstructionSetList.Name = "InstructionSetList";
            this.InstructionSetList.Size = new System.Drawing.Size(284, 466);
            this.InstructionSetList.TabIndex = 0;
            // 
            // InstructionSetLabel
            // 
            this.InstructionSetLabel.Location = new System.Drawing.Point(8, 8);
            this.InstructionSetLabel.Name = "InstructionSetLabel";
            this.InstructionSetLabel.Size = new System.Drawing.Size(216, 16);
            this.InstructionSetLabel.TabIndex = 0;
            this.InstructionSetLabel.Text = "Instruction Set";
            // 
            // GeneticProgramTreeView
            // 
            this.GeneticProgramTreeView.Location = new System.Drawing.Point(298, 24);
            this.GeneticProgramTreeView.Name = "GeneticProgramTreeView";
            this.GeneticProgramTreeView.Size = new System.Drawing.Size(486, 466);
            this.GeneticProgramTreeView.TabIndex = 1;
            this.GeneticProgramTreeView.BeforeCollapse += new System.Windows.Forms.TreeViewCancelEventHandler(this.GeneticProgramTreeView_BeforeCollapse);
            this.GeneticProgramTreeView.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.GeneticProgramTreeView_AfterSelect);
            // 
            // GeneticProgramLabel
            // 
            this.GeneticProgramLabel.Location = new System.Drawing.Point(298, 8);
            this.GeneticProgramLabel.Name = "GeneticProgramLabel";
            this.GeneticProgramLabel.Size = new System.Drawing.Size(216, 16);
            this.GeneticProgramLabel.TabIndex = 0;
            this.GeneticProgramLabel.Text = "Genetic Program";
            // 
            // LoadSolutionLabel
            // 
            this.LoadSolutionLabel.Location = new System.Drawing.Point(498, 538);
            this.LoadSolutionLabel.Name = "LoadSolutionLabel";
            this.LoadSolutionLabel.Size = new System.Drawing.Size(286, 16);
            this.LoadSolutionLabel.TabIndex = 0;
            this.LoadSolutionLabel.Text = "Load a Genetic Program Solution";
            // 
            // LoadSolutionComboBox
            // 
            this.LoadSolutionComboBox.Items.AddRange(new object[] {
            "Load your last Evolved Solution",
            "Simple Light Grouped Food Solution",
            "Simple Heavy Food Solution",
            "River Crossing Solution w/ Trees",
            "River Crossing Solution w/ Dead Ants",
            "Heavy Food with River Solution"});
            this.LoadSolutionComboBox.Location = new System.Drawing.Point(498, 554);
            this.LoadSolutionComboBox.Name = "LoadSolutionComboBox";
            this.LoadSolutionComboBox.Size = new System.Drawing.Size(286, 22);
            this.LoadSolutionComboBox.TabIndex = 6;
            this.LoadSolutionComboBox.SelectedIndexChanged += new System.EventHandler(this.LoadSolutionComboBox_SelectedIndexChanged);
            // 
            // AddNewInstructionBeforeButton
            // 
            this.AddNewInstructionBeforeButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.AddNewInstructionBeforeButton.Location = new System.Drawing.Point(11, 496);
            this.AddNewInstructionBeforeButton.Name = "AddNewInstructionBeforeButton";
            this.AddNewInstructionBeforeButton.Size = new System.Drawing.Size(305, 23);
            this.AddNewInstructionBeforeButton.TabIndex = 2;
            this.AddNewInstructionBeforeButton.Text = "Add to Program before selected instruction";
            this.AddNewInstructionBeforeButton.UseVisualStyleBackColor = false;
            this.AddNewInstructionBeforeButton.Click += new System.EventHandler(this.AddNewInstructionBeforeButton_Click);
            // 
            // RemoveInstructionButton
            // 
            this.RemoveInstructionButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.RemoveInstructionButton.Location = new System.Drawing.Point(11, 582);
            this.RemoveInstructionButton.Name = "RemoveInstructionButton";
            this.RemoveInstructionButton.Size = new System.Drawing.Size(305, 23);
            this.RemoveInstructionButton.TabIndex = 5;
            this.RemoveInstructionButton.Text = "Remove selected instruction from Program";
            this.RemoveInstructionButton.UseVisualStyleBackColor = false;
            this.RemoveInstructionButton.Click += new System.EventHandler(this.RemoveInstructionButton_Click);
            // 
            // ChangeInstructionButton
            // 
            this.ChangeInstructionButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.ChangeInstructionButton.Location = new System.Drawing.Point(11, 553);
            this.ChangeInstructionButton.Name = "ChangeInstructionButton";
            this.ChangeInstructionButton.Size = new System.Drawing.Size(305, 23);
            this.ChangeInstructionButton.TabIndex = 4;
            this.ChangeInstructionButton.Text = "Change selected instruction in Program";
            this.ChangeInstructionButton.UseVisualStyleBackColor = false;
            this.ChangeInstructionButton.Click += new System.EventHandler(this.ChangeInstructionButton_Click);
            // 
            // AddNewInstructionAfterButton
            // 
            this.AddNewInstructionAfterButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.AddNewInstructionAfterButton.Location = new System.Drawing.Point(11, 525);
            this.AddNewInstructionAfterButton.Name = "AddNewInstructionAfterButton";
            this.AddNewInstructionAfterButton.Size = new System.Drawing.Size(305, 23);
            this.AddNewInstructionAfterButton.TabIndex = 3;
            this.AddNewInstructionAfterButton.Text = "Add to Program after selected instruction";
            this.AddNewInstructionAfterButton.UseVisualStyleBackColor = false;
            this.AddNewInstructionAfterButton.Click += new System.EventHandler(this.AddNewInstructionAfterButton_Click);
            // 
            // ClearProgramButton
            // 
            this.ClearProgramButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.ClearProgramButton.Location = new System.Drawing.Point(498, 582);
            this.ClearProgramButton.Name = "ClearProgramButton";
            this.ClearProgramButton.Size = new System.Drawing.Size(286, 23);
            this.ClearProgramButton.TabIndex = 7;
            this.ClearProgramButton.Text = "Clear Program";
            this.ClearProgramButton.UseVisualStyleBackColor = false;
            this.ClearProgramButton.Click += new System.EventHandler(this.ClearProgramButton_Click);
            // 
            // WarningMessage
            // 
            this.WarningMessage.Font = new System.Drawing.Font("Verdana", 9F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.WarningMessage.ForeColor = System.Drawing.Color.Maroon;
            this.WarningMessage.Location = new System.Drawing.Point(359, 500);
            this.WarningMessage.Name = "WarningMessage";
            this.WarningMessage.Size = new System.Drawing.Size(425, 38);
            this.WarningMessage.TabIndex = 12;
            // 
            // AntProgramTab
            // 
            this.BackColor = System.Drawing.SystemColors.Control;
            this.Controls.Add(this.WarningMessage);
            this.Controls.Add(this.ClearProgramButton);
            this.Controls.Add(this.AddNewInstructionAfterButton);
            this.Controls.Add(this.ChangeInstructionButton);
            this.Controls.Add(this.RemoveInstructionButton);
            this.Controls.Add(this.AddNewInstructionBeforeButton);
            this.Controls.Add(this.LoadSolutionComboBox);
            this.Controls.Add(this.LoadSolutionLabel);
            this.Controls.Add(this.GeneticProgramLabel);
            this.Controls.Add(this.GeneticProgramTreeView);
            this.Controls.Add(this.InstructionSetLabel);
            this.Controls.Add(this.InstructionSetList);
            this.Font = new System.Drawing.Font("Verdana", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.ForeColor = System.Drawing.Color.Black;
            this.Name = "AntProgramTab";
            this.Size = new System.Drawing.Size(800, 616);
            this.ResumeLayout(false);

		}
		#endregion

        /// <summary>
        /// Private reference to the main form which gets us access to the other tabs.
        /// </summary>
        private AntForm antForm;
        /// <summary>
        ///The main form which gets us access to the other tabs.
        /// </summary>
        public AntForm AntForm
        {
            set
            {
                antForm = value;
            }
            get
            {
                return antForm;
            }
        }

        /// <summary>Private copy of the program used by this control.</summary>
        private Program program;
        /// <summary>The program currently displayed on this control.</summary>
        public Program Program
        {
            set
            {
                program = value;
            }
            get
            {
                return program;
            }
        }

        /// <summary>
        /// Fills the control that displays the list of instructions and preselect
        /// common instructions for a simple food collection problem.
        /// </summary>
		private void PopulateInstructionSetList()
		{
			foreach ( string instruction in Enum.GetNames(typeof(Tokens.Instruction)))
			{
                if (!instruction.Equals("TwoStatements"))
                {
                    InstructionSetList.Items.Add(Tokens.DisplayableInstruction(instruction));
                }
			}
		}

        /// <summary>
        /// Empty the program tree and then create the default root node.
        /// The root nodes are necessary so that the user has something to select to create a program from scratch.
        /// </summary>
        /// <param name="createFirstNoOpInstruction">Should a NoOp instruction be created as the entire program.
        /// Should be false if the tree is being reset right before loading a canned or evolved solution.</param>
        private void ResetProgramTree(bool createFirstNoOpInstruction)
        {
            GeneticProgramTreeView.Nodes.Clear();

            program = new Program();

            TreeNode root = new TreeNode(rootNodeText);
            GeneticProgramTreeView.Nodes.Add(root);
            GeneticProgramTreeView.SelectedNode = root;

            if (createFirstNoOpInstruction)
            {
                TreeNode child = new TreeNode(Tokens.DisplayableInstruction(Tokens.Instruction.NoOp));
                root.Nodes.Add(child);
            }

            root.ExpandAll();
        }

        /// <summary>
        /// Used by DisplayProgram to draw the program tree on the TreeView control.
        /// </summary>
        /// <param name="parent">The TreeView control to set.</param>
        /// <param name="node">The program node to draw.</param>
		private void DisplayProgramRecursive( TreeNode parent, ProgramNode node )
		{
            parent.Text = Tokens.DisplayableInstruction(node.Instruction);
            parent.Tag = node;

            if (Tokens.IsIfStatement(node.Instruction))
			{
				TreeNode left = new TreeNode();
				parent.Nodes.Add( left );
				DisplayProgramRecursive( left, node.Left );

                TreeNode elseStmt = new TreeNode("else");

                parent.Parent.Nodes.Add(elseStmt);

				TreeNode right = new TreeNode();
                elseStmt.Nodes.Add(right);
				DisplayProgramRecursive( right, node.Right );
			}
			if (node.Instruction == Tokens.Instruction.TwoStatements)
			{
                // left side of this dual statement uses original parent
				DisplayProgramRecursive( parent, node.Left );
                // need to make a new node at the same level for the right side.
                TreeNode second = new TreeNode();
                parent.Parent.Nodes.Add(second);
				DisplayProgramRecursive( second, node.Right );
			}
		}

        /// <summary>
        /// Displays a given program on the TreeView control.
        /// </summary>
        /// <param name="program"></param>
        public void DisplayProgram(Program program)
        {
            this.ResetProgramTree(false);

            TreeNode programParent = new TreeNode();
            GeneticProgramTreeView.Nodes[0].Nodes.Add(programParent);

            DisplayProgramRecursive(programParent, program.ProgramTree);

            GeneticProgramTreeView.ExpandAll();
            GeneticProgramTreeView.ShowPlusMinus = false;
        }

        /// <summary>
        /// Recursive function that converts the program displayed in the TreeView control  to a string.
        /// NoOp and TwoStatement tokens are ignored. 
        /// { } are used to surround statements in if and else blocks.
        /// This string can be converted back into a program using the Program constructor.
        /// </summary>
        /// <param name="parentNode"><The node to convert./param>
        /// <returns>Text for the portion of the program in the parentNode.</returns>
        public string ConvertTreeToString(TreeNode parentNode)
        {
            string programString = "";
            foreach (TreeNode node in parentNode.Nodes)
            {
                programString += node.Text;
                if (node.Nodes.Count > 0)
                {
                    programString += "{" + ConvertTreeToString(node) + "}";
                }
            }
            return programString;
        }

        /// <summary>
        /// Retrieves the program displayed in the TreeView control as a Program.
        /// </summary>
        /// <returns>The program in the TreeView control.</returns>
        public Program GetDisplayedProgram()
        {
            string programString = ConvertTreeToString(GeneticProgramTreeView.Nodes[0]);
            Program program = new Program(programString);
            return program;
        }

        /// <summary>
        /// Event fired when the user selects a new program from the combo box.
        /// Either a canned solution is loaded or it gets a program from another tab.
        /// </summary>
        /// <param name="sender">ComboBox that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void LoadSolutionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (LoadSolutionComboBox.Text.Equals("Simple Light Grouped Food Solution"))
            {
                program = new Program(Program.CannedPrograms.SimpleLightGroupedFoodSolution);
            }
            else if (LoadSolutionComboBox.Text.Equals("Simple Heavy Food Solution"))
            {
                program = new Program(Program.CannedPrograms.SimpleHeavyFoodSolution);
            }
            else if (LoadSolutionComboBox.Text.Equals("River Crossing Solution w/ Trees"))
            {
                program = new Program(Program.CannedPrograms.TreeRiverSolution);
            }
            else if (LoadSolutionComboBox.Text.Equals("River Crossing Solution w/ Dead Ants"))
            {
                program = new Program(Program.CannedPrograms.DeadAntRiverSolution);
            }
            else if (LoadSolutionComboBox.Text.Equals("River Crossing Solution"))
            {
                program = new Program(Program.CannedPrograms.DeadAntRiverSolution);
            }
            else if (LoadSolutionComboBox.Text.Equals("Heavy Food with River Solution"))
            {
                program = new Program(Program.CannedPrograms.HeavyFoodwithRiverSolution);
            }
            else if (LoadSolutionComboBox.Text.Equals("Load your last Evolved Solution"))
            {
                if (AntForm.GetProgramFromEvolveTab() != null)
                {
                    program = AntForm.GetProgramFromEvolveTab();
                }
            }

            DisplayProgram(program);
        }

        /// <summary>
        /// Event fired when the user tries to collapse the program tree.
        /// This function will supress the event.
        /// </summary>
        /// <remarks>
        /// Hiding the + and - buttons on the tree is not enough to supress the collapse event because the user can double click a tree node.
        /// </remarks>
        /// <param name="sender">Object that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void GeneticProgramTreeView_BeforeCollapse(object sender, TreeViewCancelEventArgs e)
        {
            e.Cancel = true;
        }

        /// <summary>
        /// Event fired when the user adds an instruction before the selected instruction in the program tree.
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void AddNewInstructionBeforeButton_Click(object sender, EventArgs e)
        {
            WarningMessage.Text = "";
            if (selectedTreeNode != null)
            {
                if (selectedTreeNode.Text.Equals(rootNodeText))
                {
                    WarningMessage.Text = "Cannot make any changes at " + rootNodeText;
                    return;
                }
                if (Tokens.IsElseStatement(selectedTreeNode.Text))
                {
                    WarningMessage.Text = "Cannot add a statement between an if and else statement.";
                    return;
                }

                // Create the new tree node to add
                TreeNode node = new TreeNode();
                node.Text = InstructionSetList.SelectedItem.ToString();
                selectedTreeNode.Parent.Nodes.Insert(selectedTreeNode.Index, node);

                // If the node is an if statement, we have to make 4 nodes total. If... NoOp else NoOp
                if (Tokens.IsIfStatement(InstructionSetList.SelectedItem.ToString()))
                {
                    TreeNode childNode = new TreeNode();
                    childNode.Text = Tokens.DisplayableInstruction(Tokens.Instruction.NoOp);
                    node.Nodes.Add(childNode);

                    node = new TreeNode();
                    node.Text = "else";
                    selectedTreeNode.Parent.Nodes.Insert(selectedTreeNode.Index, node);

                    childNode = new TreeNode();
                    childNode.Text = Tokens.DisplayableInstruction(Tokens.Instruction.NoOp);
                    node.Nodes.Add(childNode);

                    selectedTreeNode.Parent.ExpandAll();
                }
            }
        }

        /// <summary>
        /// Event fired when the user adds an instruction after the selected instruction in the program tree.
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void AddNewInstructionAfterButton_Click(object sender, EventArgs e)
        {
            WarningMessage.Text = "";
            if (selectedTreeNode != null)
            {
                if (selectedTreeNode.Text.Equals(rootNodeText))
                {
                    WarningMessage.Text = "Cannot make any changes at " + rootNodeText;
                    return;
                }
                if (Tokens.IsIfStatement(selectedTreeNode.Text))
                {
                    WarningMessage.Text = "Cannot add a statement between an if and else statement.";
                    return;
                }

                // Create the new tree node to add
                TreeNode node = new TreeNode();
                node.Text = InstructionSetList.SelectedItem.ToString();
                selectedTreeNode.Parent.Nodes.Insert(selectedTreeNode.Index + 1, node);

                // If the node is an if statement, we have to make 4 nodes total. If... NoOp else NoOp
                if (Tokens.IsIfStatement(InstructionSetList.SelectedItem.ToString()))
                {
                    TreeNode childNode = new TreeNode();
                    childNode.Text = Tokens.DisplayableInstruction(Tokens.Instruction.NoOp);
                    node.Nodes.Add(childNode);

                    node = new TreeNode();
                    node.Text = "else";
                    selectedTreeNode.Parent.Nodes.Insert(selectedTreeNode.Index + 2, node);

                    childNode = new TreeNode();
                    childNode.Text = Tokens.DisplayableInstruction(Tokens.Instruction.NoOp);
                    node.Nodes.Add(childNode);

                    selectedTreeNode.Parent.ExpandAll();
                }
            }
        }

        /// <summary>
        /// Event fired when the user removes an instruction from the program tree.
        /// This will remove the node, all child nodes, and also the else tree if there is one.
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event arguments.</param>
        private void RemoveInstructionButton_Click(object sender, EventArgs e)
        {
            WarningMessage.Text = "";
            if (selectedTreeNode != null)
            {
                if (selectedTreeNode.Text.Equals(rootNodeText))
                {
                    WarningMessage.Text = "Cannot make any changes at " + rootNodeText;
                    return;
                }

                TreeNode parent = selectedTreeNode.Parent;

                if (Tokens.IsElseStatement(selectedTreeNode.Text.ToString()))
                {
                    WarningMessage.Text = "Cannot remove the else part of an if statement.";
                    return;
                }
                else if (Tokens.IsIfStatement(selectedTreeNode.Text.ToString()))
                {
                    selectedTreeNode.Parent.Nodes.RemoveAt(selectedTreeNode.Index + 1); // remove the else
                    selectedTreeNode.Parent.Nodes.Remove(selectedTreeNode); // remove selected if block
                }
                else
                {
                    selectedTreeNode.Parent.Nodes.Remove(selectedTreeNode); // remove leaf statement
                }

                // Can't have a completely empty tree. Then there would be nothing to click on to add more nodes.
                if (parent.Nodes.Count == 0)
                {
                    TreeNode node = new TreeNode();
                    node.Text = Tokens.DisplayableInstruction(Tokens.Instruction.NoOp);
                    parent.Nodes.Add(node);
                }
            }
        }

        /// <summary>
        /// Event fired to changee the selected instruction in the tree.
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void ChangeInstructionButton_Click(object sender, EventArgs e)
        {
            WarningMessage.Text = "";
            if (selectedTreeNode != null)
            {
                if (selectedTreeNode.Text.Equals(rootNodeText))
                {
                    WarningMessage.Text = "Cannot make any changes at " + rootNodeText;
                    return;
                }

                // Can only replace if statements with other if statements.
                if (Tokens.IsIfStatement(InstructionSetList.SelectedItem.ToString()))
                {
                    if (!Tokens.IsIfStatement(selectedTreeNode.Text))
                    {
                        WarningMessage.Text = "Cannot replace a command with an if statement.";
                        return;
                    }
                    selectedTreeNode.Text = InstructionSetList.SelectedItem.ToString();
                }
                else // can only replace leaf nodes with other leaf nodes.
                {
                    if (Tokens.IsIfStatement(selectedTreeNode.Text))
                    {
                        WarningMessage.Text = "Cannot replace an if statement with a command.";
                        return;
                    }
                    selectedTreeNode.Text = InstructionSetList.SelectedItem.ToString();
                }
            }
        }
        
        /// <summary>
        /// Fired after the user selects a node in the tree.
        /// </summary>
        /// <remarks>
        /// The warning message is cleared.
        /// The background color of the previously selected node is cleared.
        /// The background color of the newly selected node is changed to blue.
        /// This backgrond color of the program node must be changed because the node is not really selected once another control gets focus.
        /// </remarks>
        /// <param name="sender">Object that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void GeneticProgramTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            WarningMessage.Text = "";
            if (selectedTreeNode != null)
            {
                selectedTreeNode.BackColor = Color.White;
                selectedTreeNode.ForeColor = Color.Black;
            }

            selectedTreeNode = e.Node;
            selectedTreeNode.BackColor = selectColor;
            selectedTreeNode.ForeColor = Color.White;
        }

        /// <summary>
        /// Event fired to reset the program tree.
        /// </summary>
        /// <param name="sender">Button that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        private void ClearProgramButton_Click(object sender, EventArgs e)
        {
            WarningMessage.Text = "";
            ResetProgramTree(true);
        }
	}
}
