//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: Icons.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections;
using System.Drawing;
using System.IO;

namespace AntGP
{
	/// <summary>
	/// This class caches and serves up the images used to draw the map.
	/// </summary>
	public class Icons
	{
        /// <summary>
        /// Table of all the image objects.
        /// </summary>
		private static Hashtable imageList;

        /// <summary>
        /// Default constructor loads the image files.
        /// </summary>
		public Icons()
		{
			if ( imageList == null )
			{
				imageList = new Hashtable(20);

				imageList.Add(MapItem.Item.Nothing, LoadImage("Nothing.gif"));
                imageList.Add(MapItem.Item.Nest, LoadImage("Nest.gif"));
                imageList.Add(MapItem.Item.AntWithFood, LoadImage("AntWithFood.gif"));
                imageList.Add(MapItem.Item.Ant, LoadImage("AntWithoutFood.gif"));
                imageList.Add(MapItem.Item.AntWithLeaf, LoadImage("AntWithLeaf.gif"));
                imageList.Add(MapItem.Item.DeadAnt, LoadImage("DeadAnt.gif"));
                imageList.Add(MapItem.Item.Water, LoadImage("Water.gif"));
                imageList.Add(MapItem.Item.Obstacle, LoadImage("Obstacle.gif"));
                imageList.Add(MapItem.Item.Tree, LoadImage("Tree.gif"));
                imageList.Add(MapItem.Item.Leaf, LoadImage("Leaf.gif"));
                imageList.Add(MapItem.Item.LeafOnWater, LoadImage("LeafOnWater.gif"));
                imageList.Add(MapItem.Item.Food1, LoadImage("Food1.gif"));
                imageList.Add(MapItem.Item.Food2, LoadImage("Food2.gif"));
                imageList.Add(MapItem.Item.Food3, LoadImage("Food3.gif"));
                imageList.Add(MapItem.Item.Food4, LoadImage("Food4.gif"));
                imageList.Add(MapItem.Item.Food5, LoadImage("Food5.gif"));
                imageList.Add(MapItem.Item.Food6, LoadImage("Food6.gif"));
                imageList.Add(MapItem.Item.Food7, LoadImage("Food7.gif"));
                imageList.Add(MapItem.Item.Food8, LoadImage("Food8.gif"));
                imageList.Add(MapItem.Item.Food9, LoadImage("Food9.gif"));
			}
		}

        /// <summary>
        /// Load an individual image.
        /// </summary>
        /// <param name="fileName">The file name of the image embedded in this DLL</param>
        /// <returns>The image.</returns>
		private Bitmap LoadImage( string fileName )
		{
			Stream s = this.GetType().Assembly.GetManifestResourceStream("AntGP.images." + fileName);
			Bitmap bmp = new Bitmap( s );
			return bmp;
		}

        /// <summary>
        /// Returns an image of the given map item type.
        /// </summary>
        /// <param name="icon">The item to get the image for.</param>
        /// <returns>The image</returns>
        public Image GetIcon(MapItem.Item icon)
		{
			return( (Image) imageList[icon] );
		}
        
        /// <summary>
        /// Returns an image of the given map item type at a given size.
        /// </summary>
        /// <param name="icon">The item to get the image for.</param>
        /// <param name="size">The size (width and height) of the image.</param>
        /// <returns>The image.</returns>
        public Image GetIcon(MapItem.Item icon, Size size)
		{
			Bitmap bmp = new Bitmap( GetIcon( icon ), size );
			return( (Image) bmp );
		}
	}
}
